// Vector.h: interface for the Vector class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VECTOR_H__1D4FA641_6BA6_11D5_923F_00A024900B4B__INCLUDED_)
#define AFX_VECTOR_H__1D4FA641_6BA6_11D5_923F_00A024900B4B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class	Vector {
	
private :
	
	double		_vx, _vy, _vz ;
	
public :
	
	//  Constructors
	
	Vector () ;
	
	Vector ( const double, const double = 0.0, const double = 0.0 ) ;
	
	//  Copy Constructor
	
	Vector ( const Vector& ) ;
	
	//  Destructor
	
	virtual ~Vector () ;
	
	//  Assignment
	
	Vector& operator= ( const Vector& ) ;
	
	
	//  Comparison
	
	friend int operator== ( const Vector&, const Vector& ) ;
	friend int operator!= ( const Vector&, const Vector& ) ;
	
	//  Sum, Difference, Scalar Product
	
	friend Vector operator+ ( const Vector&, const Vector& ) ;
	friend Vector operator- ( const Vector&, const Vector& ) ;
	friend Vector operator- ( const Vector& ) ;
	friend Vector operator* ( const double&, const Vector& ) ;
	friend Vector operator* ( const Vector&, const double& ) ;
	friend Vector operator/ ( const Vector&, const double& ) ;
	
	//  Immediate Sum, Difference, Scalar Product
	
	Vector& operator+= ( const Vector& ) ;
	Vector& operator-= ( const Vector& ) ;
	Vector& operator*= ( const double& ) ;
	Vector& operator/= ( const double& ) ;
	
	//  Member Functions
	//  Dot Product
	
	friend double dot ( const Vector&, const Vector& ) ;
	
	//  Cross Product
	
	friend Vector cross ( const Vector&, const Vector& ) ;
	
	//  Normalization, Make it a unit Vector
	
	void normalize () ;
	
	//  Vector Length
	
	double length () const ;
	
	//  Access Functions to get 
	//    x-coordinate, y-coordinate or
	//    z-coordinate of the vector
	
	double x () const { return _vx ; } ;
	double y () const { return _vy ; } ;
	double z () const { return _vz ; } ;
	
} ;

//  The global constant Vectors

const Vector VZero ( 0.0, 0.0, 0.0 ) ;
const Vector VX    ( 1.0, 0.0, 0.0 ) ;
const Vector VY    ( 0.0, 1.0, 0.0 ) ;
const Vector VZ    ( 0.0, 0.0, 1.0 ) ;

#endif // !defined(AFX_VECTOR_H__1D4FA641_6BA6_11D5_923F_00A024900B4B__INCLUDED_)
